//-----------------------------------------------------------------------------
// LOGGERIMPL.CPP
// By Dean Harding
// 
// Copyright (c) 2000/2001 Dean Harding
//-----------------------------------------------------------------------------
#include "internal.h"

//-----------------------------------------------------------------------------
CLoggerImpl::CLoggerImpl( TCHAR *szLogFile )
: m_szLogFile( NULL ), m_szProcName( NULL ), m_dwFormat( 0 )
{
	m_szLogFile = new TCHAR[_tcslen( szLogFile ) + 1];
	_tcscpy( m_szLogFile, szLogFile );

	SetName( NULL );
}
//-----------------------------------------------------------------------------
CLoggerImpl::~CLoggerImpl()
{
	delete[] m_szProcName;
	delete[] m_szLogFile;
}

//-----------------------------------------------------------------------------
void CLoggerImpl::Release()
{
	// wait for the queue to empty.  That's the only way we know that none of
	// our data members are still being used.
	g_LoggerQueue.WaitForEmpty();

	delete this;
}

//-----------------------------------------------------------------------------
void CLoggerImpl::SetFormat( DWORD dwFormat )
{
	m_dwFormat = dwFormat;
}
//-----------------------------------------------------------------------------
DWORD CLoggerImpl::GetFormat()
{
	return m_dwFormat;
}

//-----------------------------------------------------------------------------
void CLoggerImpl::SetName( TCHAR *szName )
{
	if( m_szProcName != NULL )
	{
		delete[] m_szProcName;
	}

	if( szName != NULL )
	{
		m_szProcName = new TCHAR[_tcslen(szName) + 1];
		_tcscpy( m_szProcName, szName );
	}
	else
	{
		TCHAR szBuffer[MAX_PATH];
		GetModuleFileName( NULL, szBuffer, MAX_PATH );
		
		TCHAR *p = (TCHAR *)(szBuffer + _tcslen(szBuffer));
		while( *(p - 1) != '\\' && p != szBuffer ) p--;

		m_szProcName = new char[_tcslen(p) + 1];
		_tcscpy( m_szProcName, p );
	}
}
//-----------------------------------------------------------------------------
TCHAR *CLoggerImpl::GetName()
{
	return m_szProcName;
}

//-----------------------------------------------------------------------------
void CLoggerImpl::Print( TCHAR *szFormat, ... )
{
	FILETIME ftTimestamp = { 0 };
	DWORD dwThreadId = 0;
	TCHAR *szBuffer = new TCHAR[LOG_MAX_LINE];

	va_list args;
	va_start( args, szFormat );
	_vstprintf( szBuffer, szFormat, args );
	va_end( args );

	if( (m_dwFormat & LOG_TIMESTAMP) != 0 )
	{
		GetSystemTimeAsFileTime( &ftTimestamp );
	}

	if( (m_dwFormat & LOG_THREADID) != 0 )
	{
		dwThreadId = GetCurrentThreadId();
	}

	CLoggerItem *pItem = new CLoggerItem( m_dwFormat, szBuffer, m_szLogFile, m_szProcName, dwThreadId, ftTimestamp );
	g_LoggerQueue.AddItem( pItem );
}


//-----------------------------------------------------------------------------
// End of LOGGERIMPL.CPP
//-----------------------------------------------------------------------------
